export declare type ProcessStatus = 'pending' | 'fulfilled' | 'rejected';
export interface Process<T = unknown> {
    id: number;
    status: ProcessStatus;
    run: () => Promise<Process<T>>;
    response?: T;
    ms?: number;
}
export interface Queue<T = unknown> {
    id: number;
    processList: Process<T>[];
    ms?: number;
}
export interface QueueOptions {
    concurrent?: number;
    withMillis?: boolean;
}
declare class ConcurrentManager<T = unknown> {
    private concurrent;
    private counter;
    private withMillis;
    private process;
    private queueList;
    private dequeueList;
    private queueSettledEvent;
    private processSettledEvent;
    constructor(initialOpts?: QueueOptions);
    setup(opts?: QueueOptions): this;
    private enqueueProcess;
    private dequeueProcess;
    onQueueSettled(callback: (data: Queue<T>) => void): this;
    onProcessSettled(callback: (data: Process<T>) => void): this;
    queue(process: (id: number) => Promise<T>): number;
    getListedQueue(): Queue<T>[];
    getListedDequeue(): Queue<T>[];
    getProcess(id: number): Process<T>;
    getListedProcess(status?: ProcessStatus): Process<T>[];
    private getMillis;
    run(): Promise<Process<T>[]>;
}
export default ConcurrentManager;
